/* Copyright 2013 Forex Capital Markets LLC

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use these files except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
#include "stdafx.h"

#include "QMDataCollection.h"
#include "QMData.h"

#include "RemoveQuotesListener.h"
#include "UpdateInstrumentsListener.h"

#include "util.h"

/** Forward declaration. */
void showHelp();
IQMDataCollection *getQuotes(quotesmgr::IQuotesManager *quotesManager, std::string instrument, int year);
void removeQuotes(quotesmgr::IQuotesManager *, char **);
void removeLocalQuotes(quotesmgr::IQuotesManager *, const std::string&, int);
void removeData(quotesmgr::IQuotesManager *quotesManager, IQMDataCollection *quotes);
void startNextRemoveTask(quotesmgr::IQuotesManager *quotesManager, IQMData *data);
void showLocalQuotes(quotesmgr::IQuotesManager *);
IQMDataCollection *prepareListOfQMData(quotesmgr::IQuotesManager *quotesManager);
void showPreparedQMDataList(IQMDataCollection *collection);

/** The sample's entry point.
 */
int main(int argc, char **argv)
{
    // use the application module path as a base path for quotes storage
    std::string cachePath = util::getCachePath(argv[0]);

    // create an instance of QuotesManager
    quotesmgr::IError *error = 0;
    O2G2Ptr<quotesmgr::IQuotesManager> quotesManager(quotesmgr::QuotesManagerFactory::create("pricehistory.fxcorporate.com",
        80, cachePath.c_str(), &error));
    if (!quotesManager)
    {
        std::string errorMessage;
        if (error)
        {
            errorMessage = error->getMessage();
            error->release();
        }
        std::cout << "Failed to create Quotes Manager instance: " << errorMessage << std::endl;
        return -1;
    }

    if (argc < 3)
    {
        showHelp();
    }
    else
    {
        removeQuotes(quotesManager, argv);
        std::cout << std::endl;
    }
    
    showLocalQuotes(quotesManager);

    return 0;
}

/** Shows help information.
 */
void showHelp()
{
    std::cout << "Remove local quotes of some instrument for the specified year." << std::endl;
    std::cout << "usage: RemoveQuotes <instrument> <year>" << std::endl;
    std::cout << "where:" << std::endl;
    std::cout << "  instrument is a specific instrument such as EUR/USD" << std::endl;
    std::cout << "  year is a specific year." << std::endl;
    std::cout << std::endl;
}

/** Remove quotes.

    @param quotesManager
        The QuotesManager instance.
    @param arguments
        Parameters of remove command.
 */
void removeQuotes(quotesmgr::IQuotesManager *quotesManager, char **arguments)
{
    std::string instrument(arguments[1]);

    int year = -1;
    std::stringstream sstr;
    sstr << arguments[2];
    sstr >> year;

    if (year <= 0)
    {
        std::cout << "Error : year must be an integer value." << std::endl;
        return;
    }

    removeLocalQuotes(quotesManager, instrument, year);
}

/** Removes quotes from local cache.

    @param quotesManager
        QuotesManager instance.
    @param instrument
        Instrument name.

    @param year
        Year.
 */
void removeLocalQuotes(quotesmgr::IQuotesManager *quotesManager, const std::string &instrument, int year)
{
    O2G2Ptr<IQMDataCollection> quotes(getQuotes(quotesManager, instrument, year));

    if (!quotes || quotes->size() == 0)
    {
        std::cout << "Error : There are no quotes to remove." << std::endl;
        return;
    }

    removeData(quotesManager, quotes);
}

/** Removes the data from cache.

    @param quotesManager
        QuotesManager instance.
    @param quotes
        Collection of quotes manager storage data.
    @param year
        Year.
 */
void removeData(quotesmgr::IQuotesManager *quotesManager, IQMDataCollection *quotes)
{
    while (quotes->size() > 0)
    {
        O2G2Ptr<IQMData> data(quotes->get(0));
        quotes->remove(0);

        startNextRemoveTask(quotesManager, data);
    }
}

/** Fills QuotesManager data for the specified instrument and year.

    @param quotesManager
        QuotesManager instance.
    @param instrument
        Instrument name.
    @param year
        Year.

    @return
        Collection of quotes manager storage data.
 */
IQMDataCollection *getQuotes(quotesmgr::IQuotesManager *quotesManager, std::string instrument, int year)
{
    O2G2Ptr<quotesmgr::IBaseTimeframes> baseTimeframes(quotesManager->getBaseTimeframes());
    O2G2Ptr<IQMDataCollection> quotes(new QMDataCollection());

    quotesmgr::IError *error = NULL;
    for(int i(0); i < baseTimeframes->size(); ++i)
    {
        std::string timeframe = baseTimeframes->get(i);
        quotesmgr::int64 size = quotesManager->getDataSize(instrument.c_str(), timeframe.c_str(), year, &error);

        if (error)
        {
            std::cout << error->getMessage() << std::endl;
            error->release();
            continue;
        }

        if (size > 0)
        {
            O2G2Ptr<IQMData> data(new QMData(instrument.c_str(), timeframe.c_str(), year, size));
            quotes->add(data);
        }
    }

    quotes->addRef();
    return quotes;
}

/** Tries to start next remove task.

    @param quotesManager
        QuotesManager instance.
    @param data
        Quotes manager storage data.
 */
void startNextRemoveTask(quotesmgr::IQuotesManager *quotesManager, IQMData *data)
{
    O2G2Ptr<RemoveQuotesListener> removeQuotesListener(new RemoveQuotesListener());
    quotesmgr::IError *error = NULL;

    O2G2Ptr<quotesmgr::IRemoveQuotesTask> removeTask(quotesManager->createRemoveQuotesTask(data->getInstrument(), 
        data->getTimeframe(), removeQuotesListener, &error));
    if (error)
    {
        std::cout << "Failed to create remove quotes task instance: " << error->getMessage() << std::endl;
        error->release();
        return;
    }

    removeTask->addYear(data->getYear());

    quotesManager->executeTask(removeTask, &error);
    if (error)
    {
        std::cout << "Failed to execute remove task: " << error->getMessage() << std::endl;
        error->release();
        return;
    }

    removeQuotesListener->waitEvents();
}

/** Shows quotes that are available in local cache.

    @param quotesManager
        QuotesManager instance.
 */
void showLocalQuotes(quotesmgr::IQuotesManager *quotesManager)
{
    // if the instruments list is not updated, update it now
    if (!quotesManager->areInstrumentsUpdated())
    {
        O2G2Ptr<UpdateInstrumentsListener> instrumentsCallback(new UpdateInstrumentsListener());

        quotesmgr::IError *error = NULL;
        O2G2Ptr<quotesmgr::IUpdateInstrumentsTask> task(quotesManager->createUpdateInstrumentsTask(instrumentsCallback, &error));
        if (error)
        {
            std::cout << "Failed to create update instruments task instance: " << error->getMessage() << std::endl;
            error->release();
            return;
        }

        quotesManager->executeTask(task, &error);
        if (error)
        {
            std::cout << "Failed to execute update instruments task: " << error->getMessage() << std::endl;
            error->release();
            return;
        }

        instrumentsCallback->waitEvents();
    }

    O2G2Ptr<IQMDataCollection> collection(prepareListOfQMData(quotesManager));
    showPreparedQMDataList(collection);
}

/** Prepares the collection of the quotes stored in the Quotes Manager cache.

    @param quotesManager
        QuotesManager instance.
    @return
        Collection of quotes manager storage data.
 */
IQMDataCollection *prepareListOfQMData(quotesmgr::IQuotesManager *quotesManager)
{
    O2G2Ptr<QMDataCollection> collection(new QMDataCollection());
    O2G2Ptr<quotesmgr::IBaseTimeframes> timeframes(quotesManager->getBaseTimeframes());

    quotesmgr::IError *error = NULL;
    O2G2Ptr<quotesmgr::IInstruments> instruments(quotesManager->getInstruments(&error));
    if (error)
    {
        std::cout << "Failed to get instruments: " << error->getMessage() << std::endl;
        error->release();

        return NULL;
    }

    for (int i = 0; i < instruments->size(); ++i)
    {
        O2G2Ptr<quotesmgr::IInstrument> instrument(instruments->get(i));

        for (int j = 0; j < timeframes->size(); ++j)
        {
            std::string timeframe = timeframes->get(j);

            const int oldestYear(util::convertYear(instrument->getOldestQuoteDate(timeframe.c_str())));
            const int latestYear(util::convertYear(instrument->getLatestQuoteDate(timeframe.c_str())));
            
            if (latestYear >= oldestYear)
            {
                for (int year = oldestYear; year <= latestYear; ++year)
                {
                    quotesmgr::int64 size = quotesManager->getDataSize(instrument->getName(), timeframe.c_str(), year);
                    if (size > 0)
                    {
                        O2G2Ptr<IQMData> data(new QMData(instrument->getName(), timeframe.c_str(), year, size));
                        collection->add(data);
                    }
                }
            }
        }
    }

    collection->addRef();
    return collection;
}

/** Show list of available quotes.

    @param collection
        Collection of quotes manager storage data.
 */
void showPreparedQMDataList(IQMDataCollection *collection)
{
    util::SummaryInfoType instrumentsInfo;
    if (!util::summariseInstrumentDataSize(collection, instrumentsInfo))
    {
        std::cout << "There are no quotes in the local storage." << std::endl;
        return;
    }

    std::cout << "Quotes in the local storage" << std::endl;
    std::for_each(instrumentsInfo.begin(), instrumentsInfo.end(), util::printInstrumentSummaryInfo);
}
